@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# :Id: $Id: latex2mathml.py 8860 2021-10-22 16:39:59Z milde $
+# :Id: $Id: latex2mathml.py 8878 2021-11-05 11:10:44Z milde $
 # :Copyright: © 2005 Jens Jørgen Mortensen [1]_
 #             © 2010, 2021 Günter Milde.
 #
@@ -13,12 +13,15 @@
 #    This file is offered as-is, without any warranty.
 #
 # .. _2-Clause BSD license: https://opensource.org/licenses/BSD-2-Clause
-
-
-"""Convert LaTex maths code into presentational MathML."""
-
-# .. [1] the original `rst2mathml.py` in `sandbox/jensj/latex_math`
 #
+# .. [1] the original `rst2mathml.py` in `sandbox/jensj/latex_math`
+
+"""Convert LaTex maths code into presentational MathML.
+
+This module is provisional:
+the API is not settled and may change with any minor Docutils version.
+"""
+
 # Usage:
 #
 # >>> from latex2mathml import *
@@ -828,7 +831,7 @@ def parse_latex_math(node, string):
             node = node.append(mo(anomalous_chars[c], **attributes))
         elif c in "/()[]|":
             node = node.append(mo(c, stretchy=False))
-        elif c in "+*=<>,.!?';@":
+        elif c in "+*=<>,.!?`';@":
             node = node.append(mo(c))
         else:
             raise SyntaxError(u'Unsupported character: "%s"' % c)
