@@ -1,4 +1,4 @@
-# $Id: DocutilsTestSupport.py 8601 2021-01-03 21:58:06Z milde $
+# $Id: DocutilsTestSupport.py 8877 2021-11-05 11:10:34Z milde $
 # Authors: David Goodger <goodger@python.org>;
 #          Garth Kidd <garth@deadlybloodyserious.com>
 # Copyright: This module has been placed in the public domain.
@@ -532,6 +532,7 @@ class RecommonmarkParserTestCase(ParserTestCase):
     settings.halt_level = 5
     settings.debug = package_unittest.debug
 
+
 class RecommonmarkParserTestSuite(ParserTestSuite):
 
     """A collection of RecommonmarkParserTestCases."""
@@ -541,6 +542,10 @@ class RecommonmarkParserTestSuite(ParserTestSuite):
     def generateTests(self, dict, dictname='totest'):
         if 'recommonmark' not in recommonmark_wrapper.Parser.supported:
             return
+        # TODO: currently the tests are too version-specific
+        import recommonmark
+        if recommonmark.__version__ != '0.4.0':
+            return
         # suppress UserWarnings from recommonmark parser
         warnings.filterwarnings('ignore', message='Unsupported.*type')
         ParserTestSuite.generateTests(self, dict, dictname='totest')
