@@ -11,3 +11,5 @@ writer_name = "latex"
 # Settings
 # use "smartquotes" transition:
 settings_overrides['smart_quotes'] = True
+settings_overrides['legacy_column_widths'] = True
+settings_overrides['use_latex_citations'] = False
