@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 
-# $Id: test_io.py 7440 2012-06-13 14:14:12Z milde $
+# $Id: test_io.py 8880 2021-11-05 11:11:18Z milde $
 # Author: Lea Wiemann <LeWiemann@gmail.com>
 # Copyright: This module has been placed in the public domain.
 
@@ -8,17 +8,15 @@
 Test module for io.py.
 """
 
-import unittest, sys
+import sys
+import unittest
+import warnings
+
 import DocutilsTestSupport              # must be imported before docutils
 from docutils import io
-from docutils._compat import b, bytes
-from docutils.error_reporting import locale_encoding
+from docutils.utils.error_reporting import locale_encoding
 from test_error_reporting import BBuf, UBuf
 
-# python 2.3
-if not hasattr(unittest.TestCase, "assertTrue"):
-    assertTrue = unittest.TestCase.failUnless
-
 class mock_stdout(UBuf):
     encoding = 'utf8'
 
@@ -54,7 +52,7 @@ class HelperTests(unittest.TestCase):
 class InputTests(unittest.TestCase):
 
     def test_bom(self):
-        input = io.StringInput(source=b('\xef\xbb\xbf foo \xef\xbb\xbf bar'),
+        input = io.StringInput(source=b'\xef\xbb\xbf foo \xef\xbb\xbf bar',
                                encoding='utf8')
         # Assert BOMs are gone.
         self.assertEqual(input.read(), u' foo  bar')
@@ -64,26 +62,26 @@ class InputTests(unittest.TestCase):
         self.assertEqual(input.read(), u'\ufeff foo \ufeff bar')
 
     def test_coding_slug(self):
-        input = io.StringInput(source=b("""\
+        input = io.StringInput(source=b"""\
 .. -*- coding: ascii -*-
 data
 blah
-"""))
+""")
         data = input.read()
         self.assertEqual(input.successful_encoding, 'ascii')
-        input = io.StringInput(source=b("""\
+        input = io.StringInput(source=b"""\
 #! python
 # -*- coding: ascii -*-
-print "hello world"
-"""))
+print("hello world")
+""")
         data = input.read()
         self.assertEqual(input.successful_encoding, 'ascii')
-        input = io.StringInput(source=b("""\
+        input = io.StringInput(source=b"""\
 #! python
 # extraneous comment; prevents coding slug from being read
 # -*- coding: ascii -*-
-print "hello world"
-"""))
+print("hello world")
+""")
         data = input.read()
         self.assertNotEqual(input.successful_encoding, 'ascii')
 
@@ -108,25 +106,40 @@ print "hello world"
         # if no encoding is given, try decoding with utf8:
         input = io.FileInput(source_path='functional/input/cyrillic.txt')
         data = input.read()
-        if sys.version_info < (3,0):
+        if sys.version_info < (3, 0):
             # in Py3k, the locale encoding is used without --input-encoding
             # skipping the heuristic
             self.assertEqual(input.successful_encoding, 'utf-8')
 
     def test_heuristics_no_utf8(self):
         # if no encoding is given and decoding with utf8 fails,
-        # use either the locale encoding (if specified) or latin1:
+        # use either the locale encoding (if specified) or latin-1:
+        if sys.version_info >= (3, 0) and locale_encoding != "utf8":
+            # in Py3k, the locale encoding is used without --input-encoding
+            # skipping the heuristic unless decoding fails.
+            return
+        probed_encodings = (locale_encoding, 'latin-1')
         input = io.FileInput(source_path='data/latin1.txt')
         data = input.read()
-        self.assertTrue(input.successful_encoding in (locale_encoding,
-                                                      'latin-1'))
+        if input.successful_encoding not in probed_encodings:
+            raise AssertionError(
+                "guessed encoding '%s' differs from probed encodings %r"
+                % (input.successful_encoding, probed_encodings))
         if input.successful_encoding == 'latin-1':
             self.assertEqual(data, u'Gr\xfc\xdfe\n')
 
+    def test_decode_unicode(self):
+        # With the special value "unicode" or "Unicode":
+        uniinput = io.Input(encoding='unicode')
+        # keep unicode instances as-is
+        self.assertEqual(uniinput.decode(u'ja'), u'ja')
+        # raise AssertionError if data is not an unicode string
+        self.assertRaises(AssertionError, uniinput.decode, b'ja')
+
 
 class OutputTests(unittest.TestCase):
 
-    bdata = b('\xfc')
+    bdata = b'\xfc'
     udata = u'\xfc'
 
     def setUp(self):
@@ -144,7 +157,7 @@ class OutputTests(unittest.TestCase):
         self.assertEqual(self.udrain.getvalue(), self.udata)
 
     def test_write_utf8(self):
-        if sys.version_info >= (3,0):
+        if sys.version_info >= (3, 0):
             fo = io.FileOutput(destination=self.udrain, encoding='utf8',
                                autoclose=False)
             fo.write(self.udata)
@@ -155,6 +168,13 @@ class OutputTests(unittest.TestCase):
             fo.write(self.udata)
             self.assertEqual(self.bdrain.getvalue(), self.udata.encode('utf8'))
 
+    def test_FileOutput_hande_io_errors_deprection_warning(self):
+        with warnings.catch_warnings(record=True) as wng:
+            warnings.simplefilter("always")
+            fo = io.FileOutput(handle_io_errors=True)
+            self.assertEqual(len(wng), 1, "Expected a DeprecationWarning.")
+            assert issubclass(wng[0].category, DeprecationWarning)
+
     # With destination in binary mode, data must be binary string
     # and is written as-is:
     def test_write_bytes(self):
@@ -164,25 +184,27 @@ class OutputTests(unittest.TestCase):
         self.assertEqual(self.bdrain.getvalue(), self.bdata)
 
     # Test for Python 3 features:
-    if sys.version_info >= (3,0):
+    if sys.version_info >= (3, 0):
         def test_write_bytes_to_stdout(self):
-            # binary data is written to destination.buffer, if the
-            # destination is sys.stdout or sys.stdin
-            backup = sys.stdout
-            sys.stdout = self.mock_stdout
-            fo = io.FileOutput(destination=sys.stdout, mode='wb',
-                               autoclose=False)
+            # try writing data to `destination.buffer`, if data is
+            # instance of `bytes` and writing to `destination` fails:
+            fo = io.FileOutput(destination=self.mock_stdout)
             fo.write(self.bdata)
             self.assertEqual(self.mock_stdout.buffer.getvalue(),
                              self.bdata)
-            sys.stdout = backup
 
-        def test_encoding_clash(self):
-            # Raise error, if given and destination encodings differ
-            # TODO: try the `write to .buffer` scheme instead?
-            self.assertRaises(ValueError,
-                              io.FileOutput, destination=self.mock_stdout,
-                               encoding='latin1')
+        def test_encoding_clash_resolved(self):
+            fo = io.FileOutput(destination=self.mock_stdout,
+                               encoding='latin1', autoclose=False)
+            fo.write(self.udata)
+            self.assertEqual(self.mock_stdout.buffer.getvalue(),
+                             self.udata.encode('latin1'))
+
+        def test_encoding_clash_nonresolvable(self):
+            del(self.mock_stdout.buffer)
+            fo = io.FileOutput(destination=self.mock_stdout,
+                               encoding='latin1', autoclose=False)
+            self.assertRaises(ValueError, fo.write, self.udata)
 
 
 if __name__ == '__main__':
