@@ -1,16 +1,18 @@
 #! /usr/bin/env python
 
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 3129 $
-# Date: $Date: 2005-03-26 17:21:28 +0100 (Sat, 26 Mar 2005) $
+# $Id: test_field_lists.py 8771 2021-06-18 18:55:08Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
 Tests for states.py.
 """
+from __future__ import absolute_import
+
+if __name__ == '__main__':
+    import __init__
+from test_parsers import DocutilsTestSupport
 
-from __init__ import DocutilsTestSupport
 
 def suite():
     s = DocutilsTestSupport.ParserTestSuite()
@@ -444,18 +446,17 @@ Nested field lists on one line:
         <field>
             <field_name>
                 Field name with \n\
-                <problematic ids="id2" refid="id1">
+                <problematic ids="problematic-1" refid="system-message-1">
                     *
                 bad inline markup
             <field_body>
-                <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+                <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
                     <paragraph>
                         Inline emphasis start-string without end-string.
                 <paragraph>
                     should generate warning.
 """],
-["""\
-Some edge cases:
+[r"""Some edge cases:
 
 :Empty:
 :Author: Me
@@ -468,6 +469,21 @@ No blank line before this paragraph.
 Field: marker is missing its open-colon.
 
 :Field marker is missing its close-colon.
+
+:Field\: names\: with\: colons\:: are possible.
+
+:\\Field\  names with backslashes\\: are possible, too.
+
+:\\: A backslash.
+
+:Not a\\\: field list.
+
+:Not a \: field list either.
+
+:\: Not a field list either.
+
+:\:
+    A definition list, not a field list.
 """,
 """\
 <document source="test data">
@@ -497,6 +513,172 @@ Field: marker is missing its open-colon.
         Field: marker is missing its open-colon.
     <paragraph>
         :Field marker is missing its close-colon.
+    <field_list>
+        <field>
+            <field_name>
+                Field: names: with: colons:
+            <field_body>
+                <paragraph>
+                    are possible.
+        <field>
+            <field_name>
+                \\Field names with backslashes\\
+            <field_body>
+                <paragraph>
+                    are possible, too.
+        <field>
+            <field_name>
+                \\
+            <field_body>
+                <paragraph>
+                    A backslash.
+    <paragraph>
+        :Not a\\: field list.
+    <paragraph>
+        :Not a : field list either.
+    <paragraph>
+        :: Not a field list either.
+    <definition_list>
+        <definition_list_item>
+            <term>
+                ::
+            <definition>
+                <paragraph>
+                    A definition list, not a field list.
+"""],
+[r"""
+:first: field
+:field:name:with:embedded:colons: unambiguous, no need for escapes
+
+..
+
+:embedded:colons: in first field name
+:field:\`:name: not interpreted text
+:field:\`name: not interpreted text
+""",
+"""\
+<document source="test data">
+    <field_list>
+        <field>
+            <field_name>
+                first
+            <field_body>
+                <paragraph>
+                    field
+        <field>
+            <field_name>
+                field:name:with:embedded:colons
+            <field_body>
+                <paragraph>
+                    unambiguous, no need for escapes
+    <comment xml:space="preserve">
+    <field_list>
+        <field>
+            <field_name>
+                embedded:colons
+            <field_body>
+                <paragraph>
+                    in first field name
+        <field>
+            <field_name>
+                field:`:name
+            <field_body>
+                <paragraph>
+                    not interpreted text
+        <field>
+            <field_name>
+                field:`name
+            <field_body>
+                <paragraph>
+                    not interpreted text
+"""],
+[r"""
+Edge cases involving embedded colons and interpreted text.
+
+Recognized as field list items:
+
+:field\:`name`: interpreted text (standard role) requires
+                escaping a leading colon in a field name
+
+:field:name: unambiguous, no need for escapes
+
+:field::name: double colons are OK, too
+
+:field:\`name`: not interpreted text
+
+:`field name`:code:: interpreted text with role in the field name
+                     works only when the role follows the text
+
+:a `complex`:code:\  field name: field body
+
+Not recognized as field list items:
+
+::code:`not a field name`: paragraph with interpreted text
+
+:code:`not a field name`: paragraph with interpreted text
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Edge cases involving embedded colons and interpreted text.
+    <paragraph>
+        Recognized as field list items:
+    <field_list>
+        <field>
+            <field_name>
+                field:
+                <title_reference>
+                    name
+            <field_body>
+                <paragraph>
+                    interpreted text (standard role) requires
+                    escaping a leading colon in a field name
+        <field>
+            <field_name>
+                field:name
+            <field_body>
+                <paragraph>
+                    unambiguous, no need for escapes
+        <field>
+            <field_name>
+                field::name
+            <field_body>
+                <paragraph>
+                    double colons are OK, too
+        <field>
+            <field_name>
+                field:`name`
+            <field_body>
+                <paragraph>
+                    not interpreted text
+        <field>
+            <field_name>
+                <literal classes="code">
+                    field name
+            <field_body>
+                <paragraph>
+                    interpreted text with role in the field name
+                    works only when the role follows the text
+        <field>
+            <field_name>
+                a \n\
+                <literal classes="code">
+                    complex
+                 field name
+            <field_body>
+                <paragraph>
+                    field body
+    <paragraph>
+        Not recognized as field list items:
+    <paragraph>
+        :
+        <literal classes="code">
+            not a field name
+        : paragraph with interpreted text
+    <paragraph>
+        <literal classes="code">
+            not a field name
+        : paragraph with interpreted text
 """],
 ]
 
