@@ -1,7 +1,7 @@
 #! /usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# $Id: test_inline_markup.py 8223 2018-07-11 12:12:34Z grubert $
+# $Id: test_inline_markup.py 8784 2021-06-30 07:48:12Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -9,8 +9,12 @@
 Tests for inline markup in docutils/parsers/rst/states.py.
 Interpreted text tests are in a separate module, test_interpreted.py.
 """
+from __future__ import absolute_import
+
+if __name__ == '__main__':
+    import __init__
+from test_parsers import DocutilsTestSupport
 
-from __init__ import DocutilsTestSupport
 
 def suite():
     s = DocutilsTestSupport.ParserTestSuite()
@@ -68,10 +72,10 @@ across lines*
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             *
         emphasis without closing asterisk
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline emphasis start-string without end-string.
 """],
@@ -142,10 +146,10 @@ what about *this**?
          (escaped, whitespace before end-string).
     <paragraph>
         However, '
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             *
         args' will trigger a warning and may be problematic.
-    <system_message backrefs="id2" ids="id1" level="2" line="13" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="13" source="test data" type="WARNING">
         <paragraph>
             Inline emphasis start-string without end-string.
     <paragraph>
@@ -356,10 +360,10 @@ u"""\
         ) but not (**) or '(** ' or x**2 or **kwargs or **
     <paragraph>
         (however, '
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             **
         kwargs' will trigger a warning and may be problematic)
-    <system_message backrefs="id2" ids="id1" level="2" line="4" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="4" source="test data" type="WARNING">
         <paragraph>
             Inline strong start-string without end-string.
 """],
@@ -385,10 +389,10 @@ Strong double asterisk: ******
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             **
         strong without closing asterisks
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline strong start-string without end-string.
 """],
@@ -510,11 +514,11 @@ and may be problematic)
          but not "``" or ``
     <paragraph>
         (however, \n\
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             ``
         standalone TeX quotes'' will trigger a warning
         and may be problematic)
-    <system_message backrefs="id2" ids="id1" level="2" line="4" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="4" source="test data" type="WARNING">
         <paragraph>
             Inline literal start-string without end-string.
 """],
@@ -535,10 +539,10 @@ Find the ```interpreted text``` in this paragraph!
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             ``
         literal without closing backquotes
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline literal start-string without end-string.
 """],
@@ -887,7 +891,7 @@ across lines`_
             across lines
 """],
 ["""\
-`phrase\`_ reference`_
+`phrase\\`_ reference`_
 """,
 """\
 <document source="test data">
@@ -905,9 +909,9 @@ Invalid phrase reference:
     <paragraph>
         Invalid phrase reference:
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             :role:`phrase reference`_
-    <system_message backrefs="id2" ids="id1" level="2" line="3" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="3" source="test data" type="WARNING">
         <paragraph>
             Mismatch: both interpreted text role prefix and reference suffix.
 """],
@@ -921,9 +925,9 @@ Invalid phrase reference:
     <paragraph>
         Invalid phrase reference:
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             `phrase reference`:role:_
-    <system_message backrefs="id2" ids="id1" level="2" line="3" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="3" source="test data" type="WARNING">
         <paragraph>
             Mismatch: both interpreted text role suffix and reference suffix.
 """],
@@ -933,13 +937,13 @@ Invalid phrase reference:
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             `
         phrase \n\
         <reference name="reference" refname="reference">
             reference
          without closing backquote
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline interpreted text or phrase reference start-string without end-string.
 """],
@@ -949,13 +953,13 @@ Invalid phrase reference:
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             `
         anonymous phrase \n\
         <reference anonymous="1" name="reference">
             reference
          without closing backquote
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline interpreted text or phrase reference start-string without end-string.
 """],
@@ -970,7 +974,7 @@ totest['embedded_URIs'] = [
     <paragraph>
         <reference name="phrase reference" refuri="http://example.com">
             phrase reference
-        <target ids="phrase-reference" names="phrase\ reference" refuri="http://example.com">
+        <target ids="phrase-reference" names="phrase\\ reference" refuri="http://example.com">
 """],
 ["""\
 `anonymous reference <http://example.com>`__
@@ -1117,7 +1121,7 @@ Escape trailing low-line char in URIs:
 ["""\
 Escape other char in URIs:
 
-`<reference\:1>`_
+`<reference\\:1>`_
 
 `<anonymous\\call>`__
 """,
@@ -1144,7 +1148,7 @@ totest['embedded_aliases'] = [
     <paragraph>
         <reference name="phrase reference" refname="alias">
             phrase reference
-        <target names="phrase\ reference" refname="alias">
+        <target names="phrase\\ reference" refname="alias">
 """],
 ["""\
 `anonymous reference <alias_>`__
@@ -1176,7 +1180,7 @@ phrase_>`__
             embedded alias across lines
 """],
 ["""\
-`embedded alias with whitespace <alias 
+`embedded alias with whitespace <alias
 long  phrase_>`__
 """,
 """\
@@ -1214,7 +1218,7 @@ long  phrase_>`__
 """\
 <document source="test data">
     <paragraph>
-        <reference name="anonymous reference" refname="aliaswith\ escaped :characters">
+        <reference name="anonymous reference" refname="aliaswith\\ escaped :characters">
             anonymous reference
 """],
 [r"""
@@ -1223,7 +1227,7 @@ long  phrase_>`__
 """\
 <document source="test data">
     <paragraph>
-        <reference name="anonymous reference" refname="aliaswith\ escaped :characters">
+        <reference name="anonymous reference" refname="aliaswith\\ escaped :characters">
             anonymous reference
 """],
 ]
@@ -1244,15 +1248,15 @@ _`Here is  a    TaRgeT` with case and spacial difficulties.
             target
     <paragraph>
         Here is \n\
-        <target ids="another-target" names="another\ target">
+        <target ids="another-target" names="another\\ target">
             another target
          in some text. And \n\
-        <target ids="yet-another-target" names="yet\ another\ target">
+        <target ids="yet-another-target" names="yet\\ another\\ target">
             yet
             another target
         , spanning lines.
     <paragraph>
-        <target ids="here-is-a-target" names="here\ is\ a\ target">
+        <target ids="here-is-a-target" names="here\\ is\\ a\\ target">
             Here is  a    TaRgeT
          with case and spacial difficulties.
 """],
@@ -1332,10 +1336,10 @@ And _`this`_ is just plain confusing.
         But this isn't a _target; targets require backquotes.
     <paragraph>
         And \n\
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             _`
         this`_ is just plain confusing.
-    <system_message backrefs="id2" ids="id1" level="2" line="3" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="3" source="test data" type="WARNING">
         <paragraph>
             Inline target start-string without end-string.
 """],
@@ -1345,10 +1349,10 @@ _`inline target without closing backquote
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             _`
         inline target without closing backquote
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline target start-string without end-string.
 """],
@@ -1361,7 +1365,7 @@ totest['footnote_reference'] = [
 """\
 <document source="test data">
     <paragraph>
-        <footnote_reference ids="id1" refname="1">
+        <footnote_reference ids="footnote-reference-1" refname="1">
             1
 """],
 ["""\
@@ -1370,7 +1374,7 @@ totest['footnote_reference'] = [
 """\
 <document source="test data">
     <paragraph>
-        <footnote_reference auto="1" ids="id1">
+        <footnote_reference auto="1" ids="footnote-reference-1">
 """],
 ["""\
 [#label]_
@@ -1378,7 +1382,7 @@ totest['footnote_reference'] = [
 """\
 <document source="test data">
     <paragraph>
-        <footnote_reference auto="1" ids="id1" refname="label">
+        <footnote_reference auto="1" ids="footnote-reference-1" refname="label">
 """],
 ["""\
 [*]_
@@ -1386,7 +1390,7 @@ totest['footnote_reference'] = [
 """\
 <document source="test data">
     <paragraph>
-        <footnote_reference auto="*" ids="id1">
+        <footnote_reference auto="*" ids="footnote-reference-1">
 """],
 ["""\
 Adjacent footnote refs are not possible: [*]_[#label]_ [#]_[2]_ [1]_[*]_
@@ -1405,7 +1409,7 @@ totest['citation_reference'] = [
 """\
 <document source="test data">
     <paragraph>
-        <citation_reference ids="id1" refname="citation">
+        <citation_reference ids="citation-reference-1" refname="citation">
             citation
 """],
 ["""\
@@ -1414,16 +1418,16 @@ totest['citation_reference'] = [
 """\
 <document source="test data">
     <paragraph>
-        <citation_reference ids="id1" refname="citation">
+        <citation_reference ids="citation-reference-1" refname="citation">
             citation
          and \n\
-        <citation_reference ids="id2" refname="cit-ation">
+        <citation_reference ids="citation-reference-2" refname="cit-ation">
             cit-ation
          and \n\
-        <citation_reference ids="id3" refname="cit.ation">
+        <citation_reference ids="citation-reference-3" refname="cit.ation">
             cit.ation
          and \n\
-        <citation_reference ids="id4" refname="cit1">
+        <citation_reference ids="citation-reference-4" refname="cit1">
             CIT1
          but not [CIT 1]_
 """],
@@ -1487,10 +1491,10 @@ reference|
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             |
         substitution reference without closing verbar
-    <system_message backrefs="id2" ids="id1" level="2" line="1" source="test data" type="WARNING">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="2" line="1" source="test data" type="WARNING">
         <paragraph>
             Inline substitution_reference start-string without end-string.
 """],
@@ -1502,6 +1506,23 @@ first | then || and finally |||
     <paragraph>
         first | then || and finally |||
 """],
+["""\
+Substitution reference with |mix|.
+
+.. |mix| replace:: text and hyperlink-reference_
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Substitution reference with \n\
+        <substitution_reference refname="mix">
+            mix
+        .
+    <substitution_definition names="mix">
+        text and \n\
+        <reference name="hyperlink-reference" refname="hyperlink-reference">
+            hyperlink-reference
+"""],
 ]
 
 totest['standalone_hyperlink'] = [
@@ -1643,7 +1664,7 @@ are not recognized): signal:noise, a:b.
         are not recognized): signal:noise, a:b.
 """],
 ["""\
-Escaped email addresses are not recognized: test\@example.org
+Escaped email addresses are not recognized: test\\@example.org
 """,
 """\
 <document source="test data">
@@ -1826,7 +1847,7 @@ inline markup separated by non-ASCII whitespace
 \xa0**NO-BREAK SPACE**\xa0, \xa0``NO-BREAK SPACE``\xa0, \xa0`NO-BREAK SPACE`\xa0,
 \u2000**EN QUAD**\u2000, \u2000``EN QUAD``\u2000, \u2000`EN QUAD`\u2000,
 \u202f**NARROW NBSP**\u202f, \u202f``NARROW NBSP``\u202f, \u202f`NARROW NBSP`\u202f,
-""",                                      
+""",
 u"""\
 <document source="test data">
     <paragraph>
