@@ -1,16 +1,18 @@
 #! /usr/bin/env python
 
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 1773 $
-# Date: $Date: 2003-12-20 19:48:23 +0100 (Sat, 20 Dec 2003) $
+# $Id: test_literal_blocks.py 8481 2020-01-31 08:17:24Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
 Tests for states.py.
 """
+from __future__ import absolute_import
+
+if __name__ == '__main__':
+    import __init__
+from test_parsers import DocutilsTestSupport
 
-from __init__ import DocutilsTestSupport
 
 def suite():
     s = DocutilsTestSupport.ParserTestSuite()
@@ -125,6 +127,46 @@ no blank line
     <paragraph>
         no blank line
 """],
+[r"""
+A paragraph\\::
+
+    A literal block.
+
+A paragraph\::
+
+    Not a literal block.
+""",
+r"""<document source="test data">
+    <paragraph>
+        A paragraph\:
+    <literal_block xml:space="preserve">
+        A literal block.
+    <paragraph>
+        A paragraph::
+    <block_quote>
+        <paragraph>
+            Not a literal block.
+"""],
+[r"""
+\\::
+
+    A literal block.
+
+\::
+
+    Not a literal block.
+""",
+r"""<document source="test data">
+    <paragraph>
+        \:
+    <literal_block xml:space="preserve">
+        A literal block.
+    <paragraph>
+        ::
+    <block_quote>
+        <paragraph>
+            Not a literal block.
+"""],
 ["""\
 A paragraph: ::
 
