@@ -1,14 +1,19 @@
 #! /usr/bin/env python
+# -*- coding: utf-8 -*-
 
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 3129 $
-# Date: $Date: 2005-03-26 17:21:28 +0100 (Sat, 26 Mar 2005) $
+# $Id: test_section_headers.py 8891 2021-11-17 19:04:15Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
-"""Tests for states.py."""
+"""
+Tests for states.py.
+"""
+from __future__ import absolute_import
+
+if __name__ == '__main__':
+    import __init__
+from test_parsers import DocutilsTestSupport
 
-from __init__ import DocutilsTestSupport
 
 def suite():
     s = DocutilsTestSupport.ParserTestSuite()
@@ -122,6 +127,20 @@ Test short underline.
         <paragraph>
             Test short underline.
 """],
+[u"""\
+à with combining varia
+======================
+
+Do not count combining chars in title column width.
+""",
+u"""\
+<document source="test data">
+    <section ids="a-with-combining-varia" names="a\u0300\\ with\\ combining\\ varia">
+        <title>
+            à with combining varia
+        <paragraph>
+            Do not count combining chars in title column width.
+"""],
 ["""\
 =====
 Title
@@ -205,7 +224,7 @@ Test long title and space normalization.
 """,
 """\
 <document source="test data">
-    <section ids="long-title" names="long title">
+    <section ids="long-title" names="long\\ title">
         <title>
             Long    Title
         <system_message level="2" line="1" source="test data" type="WARNING">
@@ -306,22 +325,22 @@ Paragraph 4.
 <document source="test data">
     <comment xml:space="preserve">
         Test return to existing, highest-level section (Title 3).
-    <section ids="title-1" names="title 1">
+    <section ids="title-1" names="title\\ 1">
         <title>
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
+        <section ids="title-2" names="title\\ 2">
             <title>
                 Title 2
             <paragraph>
                 Paragraph 2.
-    <section ids="title-3" names="title 3">
+    <section ids="title-3" names="title\\ 3">
         <title>
             Title 3
         <paragraph>
             Paragraph 3.
-        <section ids="title-4" names="title 4">
+        <section ids="title-4" names="title\\ 4">
             <title>
                 Title 4
             <paragraph>
@@ -354,22 +373,22 @@ Paragraph 4.
 <document source="test data">
     <paragraph>
         Test return to existing, highest-level section (Title 3, with overlines).
-    <section ids="title-1" names="title 1">
+    <section ids="title-1" names="title\\ 1">
         <title>
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
+        <section ids="title-2" names="title\\ 2">
             <title>
                 Title 2
             <paragraph>
                 Paragraph 2.
-    <section ids="title-3" names="title 3">
+    <section ids="title-3" names="title\\ 3">
         <title>
             Title 3
         <paragraph>
             Paragraph 3.
-        <section ids="title-4" names="title 4">
+        <section ids="title-4" names="title\\ 4">
             <title>
                 Title 4
             <paragraph>
@@ -398,22 +417,22 @@ Paragraph 4.
 <document source="test data">
     <paragraph>
         Test return to existing, higher-level section (Title 4).
-    <section ids="title-1" names="title 1">
+    <section ids="title-1" names="title\\ 1">
         <title>
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
+        <section ids="title-2" names="title\\ 2">
             <title>
                 Title 2
             <paragraph>
                 Paragraph 2.
-            <section ids="title-3" names="title 3">
+            <section ids="title-3" names="title\\ 3">
                 <title>
                     Title 3
                 <paragraph>
                     Paragraph 3.
-        <section ids="title-4" names="title 4">
+        <section ids="title-4" names="title\\ 4">
             <title>
                 Title 4
             <paragraph>
@@ -442,17 +461,17 @@ Paragraph 4.
 <document source="test data">
     <paragraph>
         Test bad subsection order (Title 4).
-    <section ids="title-1" names="title 1">
+    <section ids="title-1" names="title\\ 1">
         <title>
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
+        <section ids="title-2" names="title\\ 2">
             <title>
                 Title 2
             <paragraph>
                 Paragraph 2.
-    <section ids="title-3" names="title 3">
+    <section ids="title-3" names="title\\ 3">
         <title>
             Title 3
         <paragraph>
@@ -493,17 +512,17 @@ Paragraph 4.
 <document source="test data">
     <paragraph>
         Test bad subsection order (Title 4, with overlines).
-    <section ids="title-1" names="title 1">
+    <section ids="title-1" names="title\\ 1">
         <title>
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
+        <section ids="title-2" names="title\\ 2">
             <title>
                 Title 2
             <paragraph>
                 Paragraph 2.
-    <section ids="title-3" names="title 3">
+    <section ids="title-3" names="title\\ 3">
         <title>
             Title 3
         <paragraph>
@@ -526,7 +545,7 @@ Paragraph.
 """,
 """\
 <document source="test data">
-    <section ids="title-containing-inline-markup" names="title containing inline markup">
+    <section ids="title-containing-inline-markup" names="title\\ containing\\ inline\\ markup">
         <title>
             Title containing \n\
             <emphasis>
@@ -545,7 +564,7 @@ Paragraph.
 """,
 """\
 <document source="test data">
-    <section ids="numbered-title" names="1. numbered title">
+    <section ids="numbered-title" names="1.\\ numbered\\ title">
         <title>
             1. Numbered Title
         <paragraph>
@@ -571,7 +590,7 @@ Paragraph.
     <system_message level="2" line="3" source="test data" type="WARNING">
         <paragraph>
             Enumerated list ends without a blank line; unexpected unindent.
-    <section ids="numbered-title" names="3. numbered title">
+    <section ids="numbered-title" names="3.\\ numbered\\ title">
         <title>
             3. Numbered Title
         <paragraph>
@@ -788,17 +807,17 @@ Paragraph
         <paragraph>
             Possible incomplete section title.
             Treating the overline as ordinary text because it's so short.
-    <section dupnames="..." ids="id1">
+    <section dupnames="..." ids="section-1">
         <title>
             ...
         <system_message level="1" line="4" source="test data" type="INFO">
             <paragraph>
                 Possible incomplete section title.
                 Treating the overline as ordinary text because it's so short.
-        <section dupnames="..." ids="id2">
+        <section dupnames="..." ids="section-2">
             <title>
                 ...
-            <system_message backrefs="id2" level="1" line="5" source="test data" type="INFO">
+            <system_message backrefs="section-2" level="1" line="5" source="test data" type="INFO">
                 <paragraph>
                     Duplicate implicit target name: "...".
             <system_message level="1" line="7" source="test data" type="INFO">
@@ -809,10 +828,10 @@ Paragraph
         <paragraph>
             Possible incomplete section title.
             Treating the overline as ordinary text because it's so short.
-    <section dupnames="..." ids="id3">
+    <section dupnames="..." ids="section-3">
         <title>
             ...
-        <system_message backrefs="id3" level="1" line="8" source="test data" type="INFO">
+        <system_message backrefs="section-3" level="1" line="8" source="test data" type="INFO">
             <paragraph>
                 Duplicate implicit target name: "...".
         <paragraph>
@@ -850,7 +869,7 @@ Empty Section
 """,
 """\
 <document source="test data">
-    <section ids="empty-section" names="empty section">
+    <section ids="empty-section" names="empty\\ section">
         <title>
             Empty Section
 """],
