@@ -1,17 +1,18 @@
 #! /usr/bin/env python
 
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 3199 $
-# Date: $Date: 2005-04-09 03:32:29 +0200 (Sat, 09 Apr 2005) $
+# $Id: test_contents.py 8771 2021-06-18 18:55:08Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
 Tests for `docutils.transforms.parts.Contents` (via
 `docutils.transforms.universal.LastReaderPending`).
 """
+from __future__ import absolute_import
 
-from __init__ import DocutilsTestSupport
+if __name__ == '__main__':
+    import __init__
+from test_transforms import DocutilsTestSupport
 from docutils.transforms.references import Substitutions
 from docutils.parsers.rst import Parser
 
@@ -52,45 +53,45 @@ Paragraph 4.
         <bullet_list>
             <list_item>
                 <paragraph>
-                    <reference ids="id1" refid="title-1">
+                    <reference ids="toc-entry-1" refid="title-1">
                         Title 1
                 <bullet_list>
                     <list_item>
                         <paragraph>
-                            <reference ids="id2" refid="title-2">
+                            <reference ids="toc-entry-2" refid="title-2">
                                 Title
                                  2
                         <bullet_list>
                             <list_item>
                                 <paragraph>
-                                    <reference ids="id3" refid="title-3">
+                                    <reference ids="toc-entry-3" refid="title-3">
                                         Title
                                          3
                     <list_item>
                         <paragraph>
-                            <reference ids="id4" refid="title-4">
+                            <reference ids="toc-entry-4" refid="title-4">
                                 Title 4
-    <section ids="title-1" names="title 1">
-        <title refid="id1">
+    <section ids="title-1" names="title\\ 1">
+        <title refid="toc-entry-1">
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
+        <section ids="title-2" names="title\\ 2">
             <title>
                 <reference name="Title" refname="title">
                     Title
                  2
             <paragraph>
                 Paragraph 2.
-            <section ids="title-3" names="title 3">
-                <title refid="id3">
+            <section ids="title-3" names="title\\ 3">
+                <title refid="toc-entry-3">
                     <target ids="title" names="title">
                         Title
                      3
                 <paragraph>
                     Paragraph 3.
-        <section ids="title-4" names="title 4">
-            <title refid="id4">
+        <section ids="title-4" names="title\\ 4">
+            <title refid="toc-entry-4">
                 Title 4
             <paragraph>
                 Paragraph 4.
@@ -108,26 +109,26 @@ Paragraph 2.
 """,
 """\
 <document source="test data">
-    <topic classes="contents" ids="table-of-contents" names="table of contents">
+    <topic classes="contents" ids="table-of-contents" names="table\\ of\\ contents">
         <title>
             Table of Contents
         <bullet_list>
             <list_item>
                 <paragraph>
-                    <reference ids="id1" refid="title-1">
+                    <reference ids="toc-entry-1" refid="title-1">
                         Title 1
                 <bullet_list>
                     <list_item>
                         <paragraph>
-                            <reference ids="id2" refid="title-2">
+                            <reference ids="toc-entry-2" refid="title-2">
                                 Title 2
-    <section ids="title-1" names="title 1">
-        <title refid="id1">
+    <section ids="title-1" names="title\\ 1">
+        <title refid="toc-entry-1">
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
-            <title refid="id2">
+        <section ids="title-2" names="title\\ 2">
+            <title refid="toc-entry-2">
                 Title 2
             <paragraph>
                 Paragraph 2.
@@ -147,29 +148,29 @@ Paragraph 2.
 """,
 """\
 <document source="test data">
-    <topic classes="contents" ids="there-s-an-image-in-title-2" names="there's an image in title 2">
+    <topic classes="contents" ids="there-s-an-image-in-title-2" names="there's\\ an\\ image\\ in\\ title\\ 2">
         <title>
             There's an image in Title 2
         <bullet_list>
             <list_item>
                 <paragraph>
-                    <reference ids="id1" refid="title-1">
+                    <reference ids="toc-entry-1" refid="title-1">
                         Title 1
             <list_item>
                 <paragraph>
-                    <reference ids="id2" refid="title-2">
+                    <reference ids="toc-entry-2" refid="title-2">
                         Title 2
-    <section ids="title-1" names="title 1">
-        <title refid="id1">
+    <section ids="title-1" names="title\\ 1">
+        <title refid="toc-entry-1">
             Title 1
         <paragraph>
             Paragraph 1.
-    <section ids="title-2" names="title 2">
-        <title refid="id2">
+    <section ids="title-2" names="title\\ 2">
+        <title refid="toc-entry-2">
             <image alt="Title 2" uri="title2.png">
         <paragraph>
             Paragraph 2.
-        <substitution_definition names="Title 2">
+        <substitution_definition names="Title\\ 2">
             <image alt="Title 2" uri="title2.png">
 """],                                   # emacs cruft: "
 ["""\
@@ -200,34 +201,34 @@ Paragraph 4.
         <bullet_list>
             <list_item>
                 <paragraph>
-                    <reference ids="id1" refid="title-1">
+                    <reference ids="toc-entry-1" refid="title-1">
                         Title 1
                 <bullet_list>
                     <list_item>
                         <paragraph>
-                            <reference ids="id2" refid="title-2">
+                            <reference ids="toc-entry-2" refid="title-2">
                                 Title 2
                     <list_item>
                         <paragraph>
-                            <reference ids="id3" refid="title-4">
+                            <reference ids="toc-entry-3" refid="title-4">
                                 Title 4
-    <section ids="title-1" names="title 1">
-        <title refid="id1">
+    <section ids="title-1" names="title\\ 1">
+        <title refid="toc-entry-1">
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
-            <title refid="id2">
+        <section ids="title-2" names="title\\ 2">
+            <title refid="toc-entry-2">
                 Title 2
             <paragraph>
                 Paragraph 2.
-            <section ids="title-3" names="title 3">
+            <section ids="title-3" names="title\\ 3">
                 <title>
                     Title 3
                 <paragraph>
                     Paragraph 3.
-        <section ids="title-4" names="title 4">
-            <title refid="id3">
+        <section ids="title-4" names="title\\ 4">
+            <title refid="toc-entry-3">
                 Title 4
             <paragraph>
                 Paragraph 4.
@@ -255,38 +256,38 @@ Paragraph 4.
 """,
 """\
 <document source="test data">
-    <section ids="title-1" names="title 1">
+    <section ids="title-1" names="title\\ 1">
         <title>
             Title 1
-        <topic classes="contents" ids="contents" names="contents">
+        <topic classes="contents local" ids="contents" names="contents">
             <bullet_list>
                 <list_item>
                     <paragraph>
-                        <reference ids="id1" refid="title-2">
+                        <reference ids="toc-entry-1" refid="title-2">
                             Title 2
                     <bullet_list>
                         <list_item>
                             <paragraph>
-                                <reference ids="id2" refid="title-3">
+                                <reference ids="toc-entry-2" refid="title-3">
                                     Title 3
                 <list_item>
                     <paragraph>
-                        <reference ids="id3" refid="title-4">
+                        <reference ids="toc-entry-3" refid="title-4">
                             Title 4
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
-            <title refid="id1">
+        <section ids="title-2" names="title\\ 2">
+            <title refid="toc-entry-1">
                 Title 2
             <paragraph>
                 Paragraph 2.
-            <section ids="title-3" names="title 3">
-                <title refid="id2">
+            <section ids="title-3" names="title\\ 3">
+                <title refid="toc-entry-2">
                     Title 3
                 <paragraph>
                     Paragraph 3.
-        <section ids="title-4" names="title 4">
-            <title refid="id3">
+        <section ids="title-4" names="title\\ 4">
+            <title refid="toc-entry-3">
                 Title 4
             <paragraph>
                 Paragraph 4.
@@ -303,16 +304,16 @@ Paragraph.
 """,
 """\
 <document source="test data">
-    <topic classes="contents" ids="contents" names="contents">
+    <topic classes="contents local" ids="contents" names="contents">
         <bullet_list>
             <list_item>
                 <paragraph>
-                    <reference ids="id1" refid="section">
+                    <reference ids="toc-entry-1" refid="section">
                         Section
     <paragraph>
         Test duplicate name "Contents".
     <section ids="section" names="section">
-        <title refid="id1">
+        <title refid="toc-entry-1">
             Section
         <paragraph>
             Paragraph.
@@ -333,7 +334,7 @@ Paragraph.
         <bullet_list>
             <list_item>
                 <paragraph>
-                    <reference ids="id1" refid="section">
+                    <reference ids="toc-entry-1" refid="section">
                         Section
     <section ids="section" names="section">
         <title refid="contents">
@@ -357,7 +358,7 @@ Paragraph.
         <bullet_list>
             <list_item>
                 <paragraph>
-                    <reference ids="id1" refid="section">
+                    <reference ids="toc-entry-1" refid="section">
                         Section
     <section ids="section" names="section">
         <title>
@@ -396,7 +397,7 @@ Paragraph 3.
 """,
 """\
 <document source="test data">
-    <section ids="title-1" names="title 1">
+    <section ids="title-1" names="title\\ 1">
         <title>
             Title 1
         <paragraph>
@@ -404,24 +405,24 @@ Paragraph 3.
         <sidebar>
             <title>
                 Contents
-            <topic classes="contents" ids="contents" names="contents">
+            <topic classes="contents local" ids="contents" names="contents">
                 <bullet_list>
                     <list_item>
                         <paragraph>
-                            <reference ids="id1" refid="title-2">
+                            <reference ids="toc-entry-1" refid="title-2">
                                 Title 2
                         <bullet_list>
                             <list_item>
                                 <paragraph>
-                                    <reference ids="id2" refid="title-3">
+                                    <reference ids="toc-entry-2" refid="title-3">
                                         Title 3
-        <section ids="title-2" names="title 2">
-            <title refid="id1">
+        <section ids="title-2" names="title\\ 2">
+            <title refid="toc-entry-1">
                 Title 2
             <paragraph>
                 Paragraph 2.
-            <section ids="title-3" names="title 3">
-                <title refid="id2">
+            <section ids="title-3" names="title\\ 3">
+                <title refid="toc-entry-2">
                     Title 3
                 <paragraph>
                     Paragraph 3.
