@@ -1,21 +1,20 @@
 #! /usr/bin/env python
 
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 3149 $
-# Date: $Date: 2005-03-30 22:51:06 +0200 (Wed, 30 Mar 2005) $
+# $Id: test_hyperlinks.py 8831 2021-09-20 13:03:39Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
 Tests for docutils.transforms.references.Hyperlinks.
 """
+from __future__ import absolute_import
 
-from __init__ import DocutilsTestSupport
+if __name__ == '__main__':
+    import __init__
+from test_transforms import DocutilsTestSupport
 from docutils.transforms.references import PropagateTargets, \
      AnonymousHyperlinks, IndirectHyperlinks, ExternalTargets, \
-     InternalTargets
-
-from docutils.transforms.universal import FinalChecks
+     InternalTargets, DanglingReferences
 from docutils.parsers.rst import Parser
 
 
@@ -31,9 +30,8 @@ totest = {}
 # target/reference, direct/indirect, internal/external, and named/anonymous,
 # plus embedded URIs.
 totest['exhaustive_hyperlinks'] = ((PropagateTargets, AnonymousHyperlinks,
-                                    IndirectHyperlinks,
-                                    ExternalTargets, InternalTargets,
-                                    FinalChecks), [
+                                    IndirectHyperlinks, ExternalTargets,
+                                    InternalTargets, DanglingReferences), [
 ["""\
 direct_ external
 
@@ -142,13 +140,13 @@ circular_ indirect reference
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id2" refid="id1">
+        <problematic ids="problematic-1" refid="system-message-1">
             circular_
          indirect reference
     <target ids="circular" names="circular" refid="circular">
-    <problematic ids="id3" refid="id1">
+    <problematic ids="problematic-2 indirect" names="indirect" refid="system-message-1">
         .. _indirect: circular_
-    <system_message backrefs="id2 id3" ids="id1" level="3" line="3" source="test data" type="ERROR">
+    <system_message backrefs="problematic-1 problematic-2" ids="system-message-1" level="3" line="3" source="test data" type="ERROR">
         <paragraph>
             Indirect hyperlink target "circular" (id="circular") refers to target "indirect", forming a circular reference.
 """],
@@ -174,25 +172,25 @@ Direct internal reference: Implicit_
             Implicit
         <paragraph>
             Duplicate implicit targets.
-    <section dupnames="implicit" ids="id1">
+    <section dupnames="implicit" ids="implicit-1">
         <title>
             Implicit
-        <system_message backrefs="id1" level="1" line="7" source="test data" type="INFO">
+        <system_message backrefs="implicit-1" level="1" line="7" source="test data" type="INFO">
             <paragraph>
                 Duplicate implicit target name: "implicit".
         <paragraph>
-            <problematic ids="id3" refid="id2">
+            <problematic ids="problematic-1" refid="system-message-1">
                 indirect_
              internal
         <target ids="indirect" names="indirect" refname="implicit">
         <paragraph>
-            Direct internal reference: 
-            <problematic ids="id5" refid="id4">
+            Direct internal reference: \n\
+            <problematic ids="problematic-2" refid="system-message-2">
                 Implicit_
-    <system_message backrefs="id3" ids="id2" level="3" line="11" source="test data" type="ERROR">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="3" line="11" source="test data" type="ERROR">
         <paragraph>
             Indirect hyperlink target "indirect" (id="indirect") refers to target "implicit", which is a duplicate, and cannot be used as a unique reference.
-    <system_message backrefs="id5" ids="id4" level="3" line="13" source="test data" type="ERROR">
+    <system_message backrefs="problematic-2" ids="system-message-2" level="3" line="13" source="test data" type="ERROR">
         <paragraph>
             Duplicate target name, cannot be used as a unique reference: "implicit".
 """],
@@ -206,7 +204,7 @@ __ http://direct
     <paragraph>
         <reference anonymous="1" name="direct external" refuri="http://direct">
             direct external
-    <target anonymous="1" ids="id1" refuri="http://direct">
+    <target anonymous="1" ids="target-1" refuri="http://direct">
 """],
 ["""\
 `indirect external`__
@@ -219,7 +217,7 @@ __ xtarget_
     <paragraph>
         <reference anonymous="1" name="indirect external" refuri="http://indirect">
             indirect external
-    <target anonymous="1" ids="id1" refuri="http://indirect">
+    <target anonymous="1" ids="target-1" refuri="http://indirect">
     <target ids="xtarget" names="xtarget" refuri="http://indirect">
 """],
 ["""\
@@ -229,9 +227,9 @@ __
 """,
 """\
 <document source="test data">
-    <target anonymous="1" refid="id1">
-    <paragraph ids="id1">
-        <reference anonymous="1" name="direct internal" refid="id1">
+    <target anonymous="1" refid="target-1">
+    <paragraph ids="target-1">
+        <reference anonymous="1" name="direct internal" refid="target-1">
             direct internal
 """],
 ["""\
@@ -247,7 +245,7 @@ __ ztarget_
     <paragraph ids="ztarget" names="ztarget">
         <reference anonymous="1" name="indirect internal" refid="ztarget">
             indirect internal
-    <target anonymous="1" ids="id1" refid="ztarget">
+    <target anonymous="1" ids="target-1" refid="ztarget">
 """],
 ["""\
 .. _ztarget:
@@ -267,19 +265,49 @@ __ ztarget_
     <target dupnames="ztarget" refid="ztarget">
     <paragraph ids="ztarget">
         First
-    <system_message backrefs="id1" level="2" line="5" source="test data" type="WARNING">
+    <system_message backrefs="ztarget-1" level="2" line="5" source="test data" type="WARNING">
         <paragraph>
             Duplicate explicit target name: "ztarget".
-    <target dupnames="ztarget" refid="id1">
-    <paragraph ids="id1">
+    <target dupnames="ztarget" refid="ztarget-1">
+    <paragraph ids="ztarget-1">
         Second
     <paragraph>
-        <problematic ids="id4" refid="id3">
+        <problematic ids="problematic-1" refid="system-message-1">
             `indirect internal`__
-    <target anonymous="1" ids="id2" refname="ztarget">
-    <system_message backrefs="id4" ids="id3" level="3" line="11" source="test data" type="ERROR">
+    <target anonymous="1" ids="target-1" refname="ztarget">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="3" line="11" source="test data" type="ERROR">
         <paragraph>
-            Indirect hyperlink target (id="id2") refers to target "ztarget", which is a duplicate, and cannot be used as a unique reference.
+            Indirect hyperlink target (id="target-1") refers to target "ztarget", which is a duplicate, and cannot be used as a unique reference.
+"""],
+["""\
+The next anonymous hyperlink reference is parsed (and discarded) at
+some point, but nonetheless anonymous hyperlink references and targets
+match in this snippet.
+
+.. |invalid| replace:: anonymous__
+
+hyperlink__
+
+__ URL
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        The next anonymous hyperlink reference is parsed (and discarded) at
+        some point, but nonetheless anonymous hyperlink references and targets
+        match in this snippet.
+    <system_message level="3" line="5" source="test data" type="ERROR">
+        <paragraph>
+            Substitution definition contains illegal element <reference>:
+        <literal_block xml:space="preserve">
+            <reference anonymous="1" name="anonymous">
+                anonymous
+        <literal_block xml:space="preserve">
+            .. |invalid| replace:: anonymous__
+    <paragraph>
+        <reference anonymous="1" name="hyperlink" refuri="URL">
+            hyperlink
+    <target anonymous="1" ids="target-1" refuri="URL">
 """],
 ["""\
 An `embedded uri <http://direct>`_.
@@ -292,7 +320,7 @@ Another reference to the same `embedded URI`_.
         An \n\
         <reference name="embedded uri" refuri="http://direct">
             embedded uri
-        <target ids="embedded-uri" names="embedded uri" refuri="http://direct">
+        <target ids="embedded-uri" names="embedded\\ uri" refuri="http://direct">
         .
     <paragraph>
         Another reference to the same \n\
@@ -312,6 +340,103 @@ An `anonymous embedded uri <http://direct>`__.
         .
 """],
 ["""\
+An `embedded alias <alias_>`_.
+
+Another reference to the same `embedded alias`_.
+
+.. _alias: ham.py
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        An \n\
+        <reference name="embedded alias" refuri="ham.py">
+            embedded alias
+        <target names="embedded\\ alias" refuri="ham.py">
+        .
+    <paragraph>
+        Another reference to the same \n\
+        <reference name="embedded alias" refuri="ham.py">
+            embedded alias
+        .
+    <target ids="alias" names="alias" refuri="ham.py">
+"""],
+["""\
+An `anonymous embedded alias <redirect_>`__.
+
+.. _redirect: spam.py
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        An \n\
+        <reference name="anonymous embedded alias" refuri="spam.py">
+            anonymous embedded alias
+        .
+    <target ids="redirect" names="redirect" refuri="spam.py">
+"""],
+["""\
+An `embedded alias <alias_>`_ with unknown reference.
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        An \n\
+        <problematic ids="problematic-1" refid="system-message-2">
+            `embedded alias <alias_>`_
+        <target names="embedded\\ alias" refname="alias">
+         with unknown reference.
+    <system_message ids="system-message-1" level="3" line="1" source="test data" type="ERROR">
+        <paragraph>
+            Indirect hyperlink target "embedded alias"  refers to target "alias", which does not exist.
+    <system_message backrefs="problematic-1" ids="system-message-2" level="3" line="1" source="test data" type="ERROR">
+        <paragraph>
+            Unknown target name: "alias".
+    <system_message level="1" line="1" source="test data" type="INFO">
+        <paragraph>
+            Hyperlink target "embedded alias" is not referenced.\
+"""],
+["""\
+An embedded URI with trailing underline:
+`__init__ <http:example.py.html#__init__>`__.
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        An embedded URI with trailing underline:
+        <reference name="__init__" refuri="http:example.py.html#__init__">
+            __init__
+        .
+"""],
+["""\
+Hyperlinks with angle-bracketed text need escaping.
+
+See `Element \\<a>`_, `Element <b\\>`_, and `Element <c>\\ `_.
+
+.. _`Element <a>`:
+.. _`Element <b>`:
+.. _`Element <c>`: elements.txt
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Hyperlinks with angle-bracketed text need escaping.
+    <paragraph>
+        See \n\
+        <reference name="Element <a>" refuri="elements.txt">
+            Element <a>
+        , \n\
+        <reference name="Element <b>" refuri="elements.txt">
+            Element <b>
+        , and \n\
+        <reference name="Element <c>" refuri="elements.txt">
+            Element <c>
+        .
+    <target refid="element-a">
+    <target refid="element-b">
+    <target ids="element-c element-b element-a" names="element\\ <c> element\\ <b> element\\ <a>" refuri="elements.txt">
+"""],
+["""\
 .. _target:
 
 .. [1] Footnote; target_
@@ -319,7 +444,7 @@ An `anonymous embedded uri <http://direct>`__.
 """\
 <document source="test data">
     <target ids="target" names="target">
-    <footnote ids="id1" names="1">
+    <footnote ids="footnote-1" names="1">
         <label>
             1
         <paragraph>
@@ -347,7 +472,7 @@ An `anonymous embedded uri <http://direct>`__.
 
 totest['hyperlinks'] = ((PropagateTargets, AnonymousHyperlinks,
                          IndirectHyperlinks, ExternalTargets,
-                         InternalTargets, FinalChecks), [
+                         InternalTargets, DanglingReferences), [
 ["""\
 .. _internal hyperlink:
 
@@ -358,7 +483,7 @@ By this `internal hyperlink`_ reference.
 """\
 <document source="test data">
     <target refid="internal-hyperlink">
-    <paragraph ids="internal-hyperlink" names="internal hyperlink">
+    <paragraph ids="internal-hyperlink" names="internal\\ hyperlink">
         This paragraph referenced.
     <paragraph>
         By this \n\
@@ -381,7 +506,7 @@ The results of the transform are not visible at the XML level.
 <document source="test data">
     <target refid="chained">
     <target refid="internal-hyperlink">
-    <paragraph ids="internal-hyperlink chained" names="internal hyperlink chained">
+    <paragraph ids="internal-hyperlink chained" names="internal\\ hyperlink chained">
         This paragraph referenced.
     <paragraph>
         By this \n\
@@ -396,6 +521,24 @@ The results of the transform are not visible at the XML level.
         The results of the transform are not visible at the XML level.
 """],
 ["""\
+.. _chained:
+__ http://anonymous
+
+Anonymous__ and chained_ both refer to the same URI.
+""",
+"""\
+<document source="test data">
+    <target refid="chained">
+    <target anonymous="1" ids="target-1 chained" names="chained" refuri="http://anonymous">
+    <paragraph>
+        <reference anonymous="1" name="Anonymous" refuri="http://anonymous">
+            Anonymous
+         and \n\
+        <reference name="chained" refuri="http://anonymous">
+            chained
+         both refer to the same URI.
+"""],
+["""\
 .. _a:
 .. _b:
 
@@ -452,7 +595,7 @@ b_
 .. _a:
 .. _b:
 
-a_\ b_
+a_\\ b_
 """,
 """\
 <document source="test data">
@@ -471,7 +614,7 @@ a_\ b_
 """,
 """\
 <document source="test data">
-    <target ids="external-hyperlink" names="external hyperlink" refuri="http://uri">
+    <target ids="external-hyperlink" names="external\\ hyperlink" refuri="http://uri">
     <paragraph>
         <reference name="External hyperlink" refuri="http://uri">
             External hyperlink
@@ -483,8 +626,8 @@ a_\ b_
 """,
 """\
 <document source="test data">
-    <target ids="external-hyperlink" names="external hyperlink" refuri="http://uri">
-    <target ids="indirect-target" names="indirect target" refuri="http://uri">
+    <target ids="external-hyperlink" names="external\\ hyperlink" refuri="http://uri">
+    <target ids="indirect-target" names="indirect\\ target" refuri="http://uri">
     <system_message level="1" line="2" source="test data" type="INFO">
         <paragraph>
             Hyperlink target "indirect target" is not referenced.
@@ -499,7 +642,7 @@ and a chained_ reference too.
 """\
 <document source="test data">
     <target refid="chained">
-    <target ids="external-hyperlink chained" names="external hyperlink chained" refuri="http://uri">
+    <target ids="external-hyperlink chained" names="external\\ hyperlink chained" refuri="http://uri">
     <paragraph>
         <reference name="External hyperlink" refuri="http://uri">
             External hyperlink
@@ -517,8 +660,8 @@ and a chained_ reference too.
 """,
 """\
 <document source="test data">
-    <target ids="external-hyperlink" names="external hyperlink" refuri="http://uri">
-    <target ids="indirect-hyperlink" names="indirect hyperlink" refuri="http://uri">
+    <target ids="external-hyperlink" names="external\\ hyperlink" refuri="http://uri">
+    <target ids="indirect-hyperlink" names="indirect\\ hyperlink" refuri="http://uri">
     <paragraph>
         <reference name="Indirect hyperlink" refuri="http://uri">
             Indirect hyperlink
@@ -533,9 +676,9 @@ Chained_ `indirect hyperlink`_ reference.
 """,
 """\
 <document source="test data">
-    <target ids="external-hyperlink" names="external hyperlink" refuri="http://uri">
+    <target ids="external-hyperlink" names="external\\ hyperlink" refuri="http://uri">
     <target refuri="http://uri">
-    <target ids="indirect-hyperlink chained" names="indirect hyperlink chained" refuri="http://uri">
+    <target ids="indirect-hyperlink chained" names="indirect\\ hyperlink chained" refuri="http://uri">
     <paragraph>
         <reference name="Chained" refuri="http://uri">
             Chained
@@ -560,13 +703,13 @@ __
 """,
 """\
 <document source="test data">
-    <target anonymous="1" ids="id1" refuri="http://full">
-    <target anonymous="1" refid="id2">
-    <target anonymous="1" ids="id3 id2" refuri="http://simplified">
+    <target anonymous="1" ids="target-1" refuri="http://full">
+    <target anonymous="1" refid="target-2">
+    <target anonymous="1" ids="target-3 target-2" refuri="http://simplified">
     <target ids="external" names="external" refuri="http://indirect.external">
-    <target anonymous="1" ids="id4" refuri="http://indirect.external">
-    <target anonymous="1" refid="id5">
-    <paragraph ids="id5">
+    <target anonymous="1" ids="target-4" refuri="http://indirect.external">
+    <target anonymous="1" refid="target-5">
+    <paragraph ids="target-5">
         <reference anonymous="1" name="Full syntax anonymous external hyperlink reference" refuri="http://full">
             Full syntax anonymous external hyperlink reference
         ,
@@ -579,7 +722,7 @@ __
         <reference anonymous="1" name="indirect anonymous hyperlink reference" refuri="http://indirect.external">
             indirect anonymous hyperlink reference
         ,
-        <reference anonymous="1" name="internal anonymous hyperlink reference" refid="id5">
+        <reference anonymous="1" name="internal anonymous hyperlink reference" refid="target-5">
             internal anonymous hyperlink reference
         .
 """],
@@ -594,15 +737,15 @@ Duplicate external target_'s (different URIs):
 <document source="test data">
     <paragraph>
         Duplicate external \n\
-        <problematic ids="id3" refid="id2">
+        <problematic ids="problematic-1" refid="system-message-1">
             target_
         's (different URIs):
     <target dupnames="target" ids="target" refuri="first">
-    <system_message backrefs="id1" level="2" line="5" source="test data" type="WARNING">
+    <system_message backrefs="target-1" level="2" line="5" source="test data" type="WARNING">
         <paragraph>
             Duplicate explicit target name: "target".
-    <target dupnames="target" ids="id1" refuri="second">
-    <system_message backrefs="id3" ids="id2" level="3" line="1" source="test data" type="ERROR">
+    <target dupnames="target" ids="target-1" refuri="second">
+    <system_message backrefs="problematic-1" ids="system-message-1" level="3" line="1" source="test data" type="ERROR">
         <paragraph>
             Duplicate target name, cannot be used as a unique reference: "target".
 """],
@@ -618,10 +761,10 @@ Duplicate external targets (different URIs) without reference:
     <paragraph>
         Duplicate external targets (different URIs) without reference:
     <target dupnames="target" ids="target" refuri="first">
-    <system_message backrefs="id1" level="2" line="5" source="test data" type="WARNING">
+    <system_message backrefs="target-1" level="2" line="5" source="test data" type="WARNING">
         <paragraph>
             Duplicate explicit target name: "target".
-    <target dupnames="target" ids="id1" refuri="second">
+    <target dupnames="target" ids="target-1" refuri="second">
 """],
 ["""\
 Several__ anonymous__ hyperlinks__, but not enough targets.
@@ -631,17 +774,17 @@ __ http://example.org
 """\
 <document source="test data">
     <paragraph>
-        <problematic ids="id3" refid="id2">
+        <problematic ids="problematic-1" refid="system-message-1">
             Several__
          \n\
-        <problematic ids="id4" refid="id2">
+        <problematic ids="problematic-2" refid="system-message-1">
             anonymous__
          \n\
-        <problematic ids="id5" refid="id2">
+        <problematic ids="problematic-3" refid="system-message-1">
             hyperlinks__
         , but not enough targets.
-    <target anonymous="1" ids="id1" refuri="http://example.org">
-    <system_message backrefs="id3 id4 id5" ids="id2" level="3" source="test data" type="ERROR">
+    <target anonymous="1" ids="target-1" refuri="http://example.org">
+    <system_message backrefs="problematic-1 problematic-2 problematic-3" ids="system-message-1" level="3" source="test data" type="ERROR">
         <paragraph>
             Anonymous hyperlink mismatch: 3 references but 1 targets.
             See "backrefs" attribute for IDs.
@@ -665,11 +808,11 @@ __ http://example.org
     <target ids="external" names="external" refuri="http://uri">
     <target ids="indirect" names="indirect" refuri="http://uri">
     <target refid="internal">
-    <reference ids="internal" name="external_" names="internal" refuri="http://uri">
+    <reference ids="internal" name="external" names="internal" refuri="http://uri">
         <image uri="picture.png">
-    <reference name="indirect_" refuri="http://uri">
+    <reference name="indirect" refuri="http://uri">
         <image uri="picture.png">
-    <reference name="internal_" refid="internal">
+    <reference name="internal" refid="internal">
         <image uri="picture.png">
 """],
 ["""\
@@ -683,20 +826,20 @@ Testing an `indirect reference to the table of contents`_.
 """,
 """\
 <document source="test data">
-    <topic classes="contents" ids="table-of-contents" names="table of contents">
+    <topic classes="contents" ids="table-of-contents" names="table\\ of\\ contents">
         <title>
             Table of Contents
         <bullet_list>
             <list_item>
                 <paragraph>
-                    <reference ids="id1" refid="section">
+                    <reference ids="toc-entry-1" refid="section">
                         Section
-    <target ids="indirect-reference-to-the-table-of-contents" names="indirect reference to the table of contents" refid="table-of-contents">
+    <target ids="indirect-reference-to-the-table-of-contents" names="indirect\\ reference\\ to\\ the\\ table\\ of\\ contents" refid="table-of-contents">
     <section ids="section" names="section">
-        <title refid="id1">
+        <title refid="toc-entry-1">
             Section
         <paragraph>
-            Testing an 
+            Testing an \n\
             <reference name="indirect reference to the table of contents" refid="table-of-contents">
                 indirect reference to the table of contents
             .
@@ -712,7 +855,7 @@ Let's reference it (`explicit target`_) to avoid an irrelevant error.
 """\
 <document source="test data">
     <target refid="explicit-target">
-    <section ids="title explicit-target" names="title explicit target">
+    <section ids="title explicit-target" names="title explicit\\ target">
         <title>
             Title
         <paragraph>
@@ -745,6 +888,51 @@ Title
         <title>
             Title
 """],
+["""\
+Unknown reference_.
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Unknown \n\
+        <problematic ids="problematic-1" refid="system-message-1">
+            reference_
+        .
+    <system_message backrefs="problematic-1" ids="system-message-1" level="3" line="1" source="test data" type="ERROR">
+        <paragraph>
+            Unknown target name: "reference".
+"""],
+["""\
+Duplicate manual footnote labels, with reference ([1]_):
+
+.. [1] Footnote.
+
+.. [1] Footnote.
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Duplicate manual footnote labels, with reference (
+        <problematic ids="footnote-reference-1" refid="system-message-1">
+            [1]_
+        ):
+    <footnote dupnames="1" ids="footnote-1">
+        <label>
+            1
+        <paragraph>
+            Footnote.
+    <footnote dupnames="1" ids="footnote-2">
+        <label>
+            1
+        <system_message backrefs="footnote-2" level="2" line="5" source="test data" type="WARNING">
+            <paragraph>
+                Duplicate explicit target name: "1".
+        <paragraph>
+            Footnote.
+    <system_message backrefs="footnote-reference-1" ids="system-message-1" level="3" line="1" source="test data" type="ERROR">
+        <paragraph>
+            Duplicate target name, cannot be used as a unique reference: "1".
+"""],
 ])
 
 
