@@ -1,17 +1,18 @@
 #! /usr/bin/env python
 
-# Author: David Goodger, Dmitry Jemerov
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 3129 $
-# Date: $Date: 2005-03-26 17:21:28 +0100 (Sat, 26 Mar 2005) $
+# $Id: test_sectnum.py 8771 2021-06-18 18:55:08Z milde $
+# Authors: David Goodger <goodger@python.org>; Dmitry Jemerov
 # Copyright: This module has been placed in the public domain.
 
 """
 Tests for `docutils.transforms.parts.SectNum` (via
 `docutils.transforms.universal.LastReaderPending`).
 """
+from __future__ import absolute_import
 
-from __init__ import DocutilsTestSupport
+if __name__ == '__main__':
+    import __init__
+from test_transforms import DocutilsTestSupport
 from docutils.transforms.references import Substitutions
 from docutils.parsers.rst import Parser
 
@@ -46,28 +47,28 @@ Paragraph 4.
 """,
 u"""\
 <document source="test data">
-    <section ids="title-1" names="title 1">
+    <section ids="title-1" names="title\\ 1">
         <title auto="1">
             <generated classes="sectnum">
                 1\u00a0\u00a0\u00a0
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
+        <section ids="title-2" names="title\\ 2">
             <title auto="1">
                 <generated classes="sectnum">
                     1.1\u00a0\u00a0\u00a0
                 Title 2
             <paragraph>
                 Paragraph 2.
-            <section ids="title-3" names="title 3">
+            <section ids="title-3" names="title\\ 3">
                 <title auto="1">
                     <generated classes="sectnum">
                         1.1.1\u00a0\u00a0\u00a0
                     Title 3
                 <paragraph>
                     Paragraph 3.
-        <section ids="title-4" names="title 4">
+        <section ids="title-4" names="title\\ 4">
             <title auto="1">
                 <generated classes="sectnum">
                     1.2\u00a0\u00a0\u00a0
@@ -84,7 +85,7 @@ Paragraph 1.
 """,
 u"""\
 <document source="test data">
-    <section ids="bold-title" names="bold title">
+    <section ids="bold-title" names="bold\\ title">
         <title auto="1">
             <generated classes="sectnum">
                 1\u00a0\u00a0\u00a0
@@ -114,26 +115,26 @@ Paragraph 4.
 """,
 u"""\
 <document source="test data">
-    <section ids="title-1" names="title 1">
+    <section ids="title-1" names="title\\ 1">
         <title auto="1">
             <generated classes="sectnum">
                 1\u00a0\u00a0\u00a0
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
+        <section ids="title-2" names="title\\ 2">
             <title auto="1">
                 <generated classes="sectnum">
                     1.1\u00a0\u00a0\u00a0
                 Title 2
             <paragraph>
                 Paragraph 2.
-            <section ids="title-3" names="title 3">
+            <section ids="title-3" names="title\\ 3">
                 <title>
                     Title 3
                 <paragraph>
                     Paragraph 3.
-        <section ids="title-4" names="title 4">
+        <section ids="title-4" names="title\\ 4">
             <title auto="1">
                 <generated classes="sectnum">
                     1.2\u00a0\u00a0\u00a0
@@ -169,49 +170,49 @@ u"""\
         <bullet_list classes="auto-toc">
             <list_item>
                 <paragraph>
-                    <reference ids="id1" refid="title-1">
+                    <reference ids="toc-entry-1" refid="title-1">
                         <generated classes="sectnum">
                             1\u00a0\u00a0\u00a0
                         Title 1
                 <bullet_list classes="auto-toc">
                     <list_item>
                         <paragraph>
-                            <reference ids="id2" refid="title-2">
+                            <reference ids="toc-entry-2" refid="title-2">
                                 <generated classes="sectnum">
                                     1.1\u00a0\u00a0\u00a0
                                 Title 2
                         <bullet_list>
                             <list_item>
                                 <paragraph>
-                                    <reference ids="id3" refid="title-3">
+                                    <reference ids="toc-entry-3" refid="title-3">
                                         Title 3
                     <list_item>
                         <paragraph>
-                            <reference ids="id4" refid="title-4">
+                            <reference ids="toc-entry-4" refid="title-4">
                                 <generated classes="sectnum">
                                     1.2\u00a0\u00a0\u00a0
                                 Title 4
-    <section ids="title-1" names="title 1">
-        <title auto="1" refid="id1">
+    <section ids="title-1" names="title\\ 1">
+        <title auto="1" refid="toc-entry-1">
             <generated classes="sectnum">
                 1\u00a0\u00a0\u00a0
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
-            <title auto="1" refid="id2">
+        <section ids="title-2" names="title\\ 2">
+            <title auto="1" refid="toc-entry-2">
                 <generated classes="sectnum">
                     1.1\u00a0\u00a0\u00a0
                 Title 2
             <paragraph>
                 Paragraph 2.
-            <section ids="title-3" names="title 3">
-                <title refid="id3">
+            <section ids="title-3" names="title\\ 3">
+                <title refid="toc-entry-3">
                     Title 3
                 <paragraph>
                     Paragraph 3.
-        <section ids="title-4" names="title 4">
-            <title auto="1" refid="id4">
+        <section ids="title-4" names="title\\ 4">
+            <title auto="1" refid="toc-entry-4">
                 <generated classes="sectnum">
                     1.2\u00a0\u00a0\u00a0
                 Title 4
@@ -240,28 +241,28 @@ Paragraph 4.
 """,
 u"""\
 <document source="test data">
-    <section ids="title-1" names="title 1">
+    <section ids="title-1" names="title\\ 1">
         <title auto="1">
             <generated classes="sectnum">
                 Arbitrary-1\u00a0\u00a0\u00a0
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
+        <section ids="title-2" names="title\\ 2">
             <title auto="1">
                 <generated classes="sectnum">
                     Arbitrary-1.1\u00a0\u00a0\u00a0
                 Title 2
             <paragraph>
                 Paragraph 2.
-            <section ids="title-3" names="title 3">
+            <section ids="title-3" names="title\\ 3">
                 <title auto="1">
                     <generated classes="sectnum">
                         Arbitrary-1.1.1\u00a0\u00a0\u00a0
                     Title 3
                 <paragraph>
                     Paragraph 3.
-        <section ids="title-4" names="title 4">
+        <section ids="title-4" names="title\\ 4">
             <title auto="1">
                 <generated classes="sectnum">
                     Arbitrary-1.2\u00a0\u00a0\u00a0
@@ -291,28 +292,28 @@ Paragraph 4.
 """,
 u"""\
 <document source="test data">
-    <section ids="title-1" names="title 1">
+    <section ids="title-1" names="title\\ 1">
         <title auto="1">
             <generated classes="sectnum">
                 3\u00a0\u00a0\u00a0
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
+        <section ids="title-2" names="title\\ 2">
             <title auto="1">
                 <generated classes="sectnum">
                     3.1\u00a0\u00a0\u00a0
                 Title 2
             <paragraph>
                 Paragraph 2.
-            <section ids="title-3" names="title 3">
+            <section ids="title-3" names="title\\ 3">
                 <title auto="1">
                     <generated classes="sectnum">
                         3.1.1\u00a0\u00a0\u00a0
                     Title 3
                 <paragraph>
                     Paragraph 3.
-        <section ids="title-4" names="title 4">
+        <section ids="title-4" names="title\\ 4">
             <title auto="1">
                 <generated classes="sectnum">
                     3.2\u00a0\u00a0\u00a0
@@ -344,28 +345,28 @@ Paragraph 4.
 """,
 u"""\
 <document source="test data">
-    <section ids="title-1" names="title 1">
+    <section ids="title-1" names="title\\ 1">
         <title auto="1">
             <generated classes="sectnum">
                 (5.9.3)\u00a0\u00a0\u00a0
             Title 1
         <paragraph>
             Paragraph 1.
-        <section ids="title-2" names="title 2">
+        <section ids="title-2" names="title\\ 2">
             <title auto="1">
                 <generated classes="sectnum">
                     (5.9.3.1)\u00a0\u00a0\u00a0
                 Title 2
             <paragraph>
                 Paragraph 2.
-            <section ids="title-3" names="title 3">
+            <section ids="title-3" names="title\\ 3">
                 <title auto="1">
                     <generated classes="sectnum">
                         (5.9.3.1.1)\u00a0\u00a0\u00a0
                     Title 3
                 <paragraph>
                     Paragraph 3.
-        <section ids="title-4" names="title 4">
+        <section ids="title-4" names="title\\ 4">
             <title auto="1">
                 <generated classes="sectnum">
                     (5.9.3.2)\u00a0\u00a0\u00a0
